@@ -31,7 +31,7 @@ new_rcrd <- function(fields, ..., class = character()) {
 
 #' @export
 vec_proxy.vctrs_rcrd <- function(x, ...) {
-  new_data_frame(unclass(x))
+  new_data_frame(x)
 }
 #' @export
 vec_restore.vctrs_rcrd <- function(x, to, ...) {
@@ -40,6 +40,12 @@ vec_restore.vctrs_rcrd <- function(x, to, ...) {
   x
 }
 
+#' @export
+vec_proxy_equal.vctrs_rcrd <- function(x, ...) {
+  # Recursively proxy using a data frame
+  vec_proxy_equal(new_data_frame(x))
+}
+
 #' @export
 length.vctrs_rcrd <- function(x) {
   vec_size(x)
@@ -50,6 +56,15 @@ names.vctrs_rcrd <- function(x) {
   NULL
 }
 
+#' @export
+`names<-.vctrs_rcrd` <- function(x, value) {
+  if (is_null(value)) {
+    x
+  } else {
+    abort("Can't assign names to a <vctrs_rcrd>.")
+  }
+}
+
 #' @export
 format.vctrs_rcrd <- function(x, ...) {
   if (inherits(x, "vctrs_foobar")) {
@@ -130,13 +145,6 @@ rep.vctrs_rcrd <- function(x, ...) {
 
 # Equality and ordering ---------------------------------------------------
 
-# FIXME
-
-#' @export
-vec_proxy_compare.vctrs_rcrd <- function(x, ...) {
-  new_data_frame(vec_data(x), n = length(x))
-}
-
 #' @export
 vec_math.vctrs_rcrd <- function(.fn, .x, ...) {
   stop_unsupported(.x, "vec_math")
