@@ -256,7 +256,7 @@ new_rational <- function(n = integer(), d = integer()) {
 }
 
 ## -----------------------------------------------------------------------------
-rational <- function(n, d) {
+rational <- function(n = integer(), d = integer()) {
   c(n, d) %<-% vec_cast_common(n, d, .to = integer())
   c(n, d) %<-% vec_recycle_common(n, d)
 
@@ -273,6 +273,16 @@ length(x)
 fields(x)
 field(x, "n")
 
+## ---- error = TRUE------------------------------------------------------------
+x
+
+str(x)
+
+## -----------------------------------------------------------------------------
+vec_data(x)
+
+str(vec_data(x))
+
 ## -----------------------------------------------------------------------------
 format.vctrs_rational <- function(x, ...) {
   n <- field(x, "n")
@@ -359,13 +369,17 @@ x == rational(1, 1)
 unique(x)
 
 ## -----------------------------------------------------------------------------
-sort(x)
+rational(1, 2) < rational(2, 3)
+rational(2, 4) < rational(2, 3)
 
 ## -----------------------------------------------------------------------------
 vec_proxy_compare.vctrs_rational <- function(x, ...) {
   field(x, "n") / field(x, "d")
 }
 
+rational(2, 4) < rational(2, 3)
+
+## -----------------------------------------------------------------------------
 sort(x)
 
 ## -----------------------------------------------------------------------------
@@ -404,7 +418,7 @@ obj_print_data.vctrs_poly <- function(x, ...) {
   print(format(x), quote = FALSE)
 }
 
-p <- poly(1, c(1, 0, 1), c(1, 0, 0, 0, 2))
+p <- poly(1, c(1, 0, 0, 0, 2), c(1, 0, 1))
 p
 
 ## -----------------------------------------------------------------------------
@@ -416,7 +430,7 @@ p[[2]]
 p == poly(c(1, 0, 1))
 
 ## ---- error = TRUE------------------------------------------------------------
-sort(p)
+p < p[2]
 
 ## -----------------------------------------------------------------------------
 vec_proxy_compare.vctrs_poly <- function(x, ...) {
@@ -431,8 +445,18 @@ vec_proxy_compare.vctrs_poly <- function(x, ...) {
   as.data.frame(do.call(rbind, full))
 }
 
-sort(poly(3, 2, 1))
-sort(poly(1, c(1, 0, 0), c(1, 0)))
+p < p[2]
+
+## -----------------------------------------------------------------------------
+sort(p)
+sort(p[c(1:3, 1:2)])
+
+## -----------------------------------------------------------------------------
+vec_proxy_order.vctrs_poly <- function(x, ...) {
+  vec_proxy_compare(x, ...)
+}
+
+sort(p)
 
 ## -----------------------------------------------------------------------------
 vec_arith.MYCLASS <- function(op, x, y, ...) {
@@ -597,18 +621,16 @@ is_percent <- function(x) {
 #    "prcnt"
 #  }
 
-## -----------------------------------------------------------------------------
-#' @export
-vec_ptype2.vctrs_percent.vctrs_percent <- function(x, y, ...) new_percent()
-#' @export
-vec_ptype2.double.vctrs_percent <- function(x, y, ...) double()
-
-## ----eval = FALSE-------------------------------------------------------------
+## ---- eval = FALSE------------------------------------------------------------
+#  #' @export
+#  vec_ptype2.vctrs_percent.vctrs_percent <- function(x, y, ...) new_percent()
+#  #' @export
+#  vec_ptype2.double.vctrs_percent <- function(x, y, ...) double()
+#  
 #  #' @export
 #  vec_cast.pizza_percent.pizza_percent <- function(x, to, ...) x
 #  #' @export
 #  vec_cast.pizza_percent.double <- function(x, to, ...) percent(x)
-#  #' @method vec_cast.double pizza_percent
 #  #' @export
 #  vec_cast.double.pizza_percent <- function(x, to, ...) vec_data(x)
 
