@@ -1,6 +1,4 @@
 #include "vctrs.h"
-#include "ptype2.h"
-#include "utils.h"
 
 static SEXP levels_union(SEXP x, SEXP y);
 
@@ -13,11 +11,11 @@ SEXP fct_ptype2(const struct ptype2_opts* opts) {
   SEXP y_levels = Rf_getAttrib(y, R_LevelsSymbol);
 
   if (TYPEOF(x_levels) != STRSXP) {
-    stop_corrupt_factor_levels(x, opts->x_arg);
+    stop_corrupt_factor_levels(x, opts->p_x_arg);
   }
 
   if (TYPEOF(y_levels) != STRSXP) {
-    stop_corrupt_factor_levels(y, opts->y_arg);
+    stop_corrupt_factor_levels(y, opts->p_y_arg);
   }
 
   // Quick early exit for identical levels pointing to the same SEXP
@@ -58,7 +56,7 @@ SEXP ord_ptype2_validate(SEXP x,
 
 // [[ include("type-factor.h") ]]
 SEXP ord_ptype2(const struct ptype2_opts* opts) {
-  SEXP levels = PROTECT(ord_ptype2_validate(opts->x, opts->y, opts->x_arg, opts->y_arg, false));
+  SEXP levels = PROTECT(ord_ptype2_validate(opts->x, opts->y, opts->p_x_arg, opts->p_y_arg, false));
   SEXP out = new_empty_ordered(levels);
   return UNPROTECT(1), out;
 }
@@ -256,7 +254,7 @@ SEXP fct_as_factor(SEXP x,
 
 // [[ include("factor.h") ]]
 SEXP ord_as_ordered(const struct cast_opts* opts) {
-  ord_ptype2_validate(opts->x, opts->to, opts->x_arg, opts->to_arg, true);
+  ord_ptype2_validate(opts->x, opts->to, opts->p_x_arg, opts->p_to_arg, true);
   return opts->x;
 }
 
@@ -354,7 +352,7 @@ static SEXP fct_as_factor_impl(SEXP x, SEXP x_levels, SEXP to_levels, bool* loss
 
 static void init_factor(SEXP x, SEXP levels) {
   if (TYPEOF(x) != INTSXP) {
-    stop_internal("init_factor", "Only integers can be made into factors.");
+    r_stop_internal("Only integers can be made into factors.");
   }
 
   Rf_setAttrib(x, R_LevelsSymbol, levels);
@@ -363,7 +361,7 @@ static void init_factor(SEXP x, SEXP levels) {
 
 static void init_ordered(SEXP x, SEXP levels) {
   if (TYPEOF(x) != INTSXP) {
-    stop_internal("init_ordered", "Only integers can be made into ordered factors.");
+    r_stop_internal("Only integers can be made into ordered factors.");
   }
 
   Rf_setAttrib(x, R_LevelsSymbol, levels);
