@@ -81,6 +81,11 @@ test_that("can compare data frames with data frame columns", {
   expect_equal(vec_compare(df1, df2), -1)
 })
 
+test_that("can compare data frames with 0 columns", {
+  x <- new_data_frame(n = 2L)
+  expect_identical(vec_compare(x, x), c(0L, 0L))
+})
+
 test_that("C code doesn't crash with bad inputs", {
   df <- data.frame(x = c(1, 1, 1), y = c(-1, 0, 1))
 
@@ -98,7 +103,8 @@ test_that("C code doesn't crash with bad inputs", {
 
 test_that("xtfrm.vctrs_vctr works for variety of base classes", {
   df <- data.frame(x = c(NA, 1, 1), y = c(1, 2, 1))
-  expect_equal(xtfrm.vctrs_vctr(df), c(3, 2, 1))
+  # Internally uses `vec_rank()`, which propagates rows if not "complete"
+  expect_equal(xtfrm.vctrs_vctr(df), c(NA, 2, 1))
 
   x <- c(2, 3, 1)
   expect_equal(xtfrm.vctrs_vctr(x), x)
@@ -194,11 +200,6 @@ test_that("vec_proxy_order() works on deeply nested lists", {
   expect_identical(vec_proxy_order(df2), data_frame(x = c(1L, 2L, 1L), y = 1:3))
 })
 
-test_that("error is thrown with data frames with 0 columns", {
-  x <- new_data_frame(n = 1L)
-  expect_error(vec_compare(x, x), "data frame with zero columns")
-})
-
 test_that("error is thrown when comparing lists", {
   expect_error(vec_compare(list(), list()), class = "vctrs_error_unsupported")
   expect_error(.Call(vctrs_compare, list(), list(), FALSE), "Can't compare lists")
