@@ -1,8 +1,7 @@
-context("test-shape")
 
 # common shape ------------------------------------------------------------
 
-test_that("recycling rules applied", {
+test_that("vec_shape2() applies recycling rules", {
   expect_equal(vec_shape2(shaped_int(1, 5, 5), shaped_int(1)),       c(0L, 5L, 5L))
   expect_equal(vec_shape2(shaped_int(1),       shaped_int(1, 5, 5)), c(0L, 5L, 5L))
   expect_equal(vec_shape2(shaped_int(1, 1),    shaped_int(1, 5, 5)), c(0L, 5L, 5L))
@@ -16,21 +15,26 @@ test_that("recycling rules applied", {
 })
 
 test_that("incompatible shapes throw errors", {
-  verify_errors({
-    expect_error(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1)), class = "vctrs_error_incompatible_type")
-    expect_error(vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5)), class = "vctrs_error_incompatible_type")
+  expect_snapshot({
+    (expect_error(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1)), class = "vctrs_error_incompatible_type"))
+    (expect_error(vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5)), class = "vctrs_error_incompatible_type"))
   })
 })
 
 test_that("can override error args", {
-  verify_errors({
-    expect_error(
+  expect_snapshot({
+    (expect_error(
       vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1), x_arg = "foo", y_arg = "bar"),
       class = "vctrs_error_incompatible_type"
-    )
+    ))
   })
 })
 
+test_that("vec_shape2() evaluates arg lazily", {
+  expect_silent(vec_shape2(shaped_int(1, 5, 5), shaped_int(1), x_arg = print("oof")))
+  expect_silent(vec_shape2(shaped_int(1, 5, 5), shaped_int(1), y_arg = print("oof")))
+})
+
 # broadcasting -------------------------------------------------------------
 
 test_that("can broadcast to higher dimension, but not lower", {
@@ -51,7 +55,7 @@ test_that("can broadcast to higher dimension, but not lower", {
   )
 })
 
-test_that("recycling rules applied", {
+test_that("shape_broadcast_() applies recycling rules", {
   expect_equal(
     shape_broadcast_(array(1:4, c(1, 1, 4)), shaped_int(0, 4, 4))[1, , ],
     matrix(1:4, 4, 4, byrow = TRUE)
@@ -79,16 +83,3 @@ test_that("recycling rules applied", {
     class = "vctrs_error_incompatible_type"
   )
 })
-
-# --------------------------------------------------------------------------
-
-test_that("shape errors have informative output", {
-  verify_output(test_path("error", "test-shape.txt"), {
-    "# incompatible shapes throw errors"
-    vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1))
-    vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5))
-
-    "# can override error args"
-    vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1), x_arg = "foo", y_arg = "bar")
-  })
-})
