@@ -46,7 +46,7 @@ test_that("data frames are split rowwise", {
   expect_equal(vec_chop(x), result)
 })
 
-test_that("data frame row names are kept", {
+test_that("vec_chop() keeps data frame row names", {
   x <- data_frame(x = 1:2, y = c("a", "b"))
   rownames(x) <- c("r1", "r2")
   result <- lapply(vec_chop(x), rownames)
@@ -192,6 +192,26 @@ test_that("vec_chop() with data frame proxies always uses the proxy's length inf
   expect_identical(proxy_deref(result2), proxy_deref(expect2))
 })
 
+test_that("ALTREP objects always generate materialized chops (#1450)", {
+  skip_if(getRversion() <= "3.5.0")
+
+  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))
+
+  # `x` starts in compact form
+  expect_false(.Call(vctrs_altrep_rle_is_materialized, x))
+
+  result <- vec_chop(x)
+
+  # Chopping materializes `x`
+  expect_true(.Call(vctrs_altrep_rle_is_materialized, x))
+
+  # And chopped elements are not ALTREP vectors
+  expect_false(any(map_lgl(result, is_altrep)))
+
+  expect <- vec_chop(c(rep("foo", 10), rep("bar", 5)))
+  expect_identical(result, expect)
+})
+
 # vec_chop + compact_seq --------------------------------------------------
 
 # `start` is 0-based
@@ -204,7 +224,7 @@ test_that("can chop base vectors with compact seqs", {
   expect_identical(vec_chop_seq(dbl(1, 2, 3), start, size), list(dbl(2, 3)))
   expect_identical(vec_chop_seq(cpl(1, 2, 3), start, size), list(cpl(2, 3)))
   expect_identical(vec_chop_seq(chr("1", "2", "3"), start, size), list(chr("2", "3")))
-  expect_identical(vec_chop_seq(bytes(1, 2, 3), start, size), list(bytes(2, 3)))
+  expect_identical(vec_chop_seq(raw2(1, 2, 3), start, size), list(raw2(2, 3)))
   expect_identical(vec_chop_seq(list(1, 2, 3), start, size), list(list(2, 3)))
 })
 
@@ -251,6 +271,11 @@ test_that("`x` and `indices` must be lists of the same size", {
   expect_error(vec_unchop(list(1, 2), list(1)), "`x` and `indices` must be lists of the same size")
 })
 
+test_that("can unchop with an AsIs list (#1463)", {
+  x <- I(list(1, 2))
+  expect_identical(vec_unchop(x), c(1, 2))
+})
+
 test_that("can unchop empty vectors", {
   expect_null(vec_unchop(list()))
   expect_null(vec_unchop(list(), list()))
@@ -405,7 +430,7 @@ test_that("not all inputs have to be named", {
   expect_named(vec_unchop(x, indices), c("", "a", "c"))
 })
 
-test_that("data frame row names are kept", {
+test_that("vec_unchop() keeps data frame row names", {
   df1 <- data.frame(x = 1:2, row.names = c("r1", "r2"))
   df2 <- data.frame(x = 3:4, row.names = c("r3", "r4"))
 
@@ -467,6 +492,8 @@ test_that("index values are validated", {
 })
 
 test_that("name repair is respected and happens after ordering according to `indices`", {
+  local_name_repair_quiet()
+
   x <- list(c(a = 1), c(a = 2))
   indices <- list(2, 1)
 
@@ -475,15 +502,15 @@ test_that("name repair is respected and happens after ordering according to `ind
 })
 
 test_that("vec_unchop() errors on unsupported location values", {
-  verify_errors({
-    expect_error(
+  expect_snapshot({
+    (expect_error(
       vec_unchop(list(1, 2), list(c(1, 2), 0)),
       class = "vctrs_error_subscript_type"
-    )
-    expect_error(
+    ))
+    (expect_error(
       vec_unchop(list(1), list(-1)),
       class = "vctrs_error_subscript_type"
-    )
+    ))
   })
 })
 
@@ -499,18 +526,18 @@ test_that("vec_unchop() works with simple homogeneous foreign S3 classes", {
 })
 
 test_that("vec_unchop() fails with complex foreign S3 classes", {
-  verify_errors({
+  expect_snapshot({
     x <- structure(foobar(1), attr_foo = "foo")
     y <- structure(foobar(2), attr_bar = "bar")
-    expect_error(vec_unchop(list(x, y)), class = "vctrs_error_incompatible_type")
+    (expect_error(vec_unchop(list(x, y)), class = "vctrs_error_incompatible_type"))
   })
 })
 
 test_that("vec_unchop() fails with complex foreign S4 classes", {
-  verify_errors({
+  expect_snapshot({
     joe <- .Counts(c(1L, 2L), name = "Joe")
     jane <- .Counts(3L, name = "Jane")
-    expect_error(vec_unchop(list(joe, jane)), class = "vctrs_error_incompatible_type")
+    (expect_error(vec_unchop(list(joe, jane)), class = "vctrs_error_incompatible_type"))
   })
 })
 
@@ -617,11 +644,11 @@ test_that("vec_unchop() falls back for S4 classes with a registered c() method",
   joe <- .Counts(c(1L, 2L), name = "Joe")
   jane <- .Counts(3L, name = "Jane")
 
-  verify_errors({
-    expect_error(
+  expect_snapshot({
+    (expect_error(
       vec_unchop(list(joe, 1, jane), list(c(1, 2), 3, 4)),
       class = "vctrs_error_incompatible_type"
-    )
+    ))
   })
 
   local_c_counts()
@@ -649,18 +676,18 @@ test_that("vec_unchop() falls back for S4 classes with a registered c() method",
 })
 
 test_that("vec_unchop() fallback doesn't support `name_spec` or `ptype`", {
-  verify_errors({
+  expect_snapshot({
     foo <- structure(foobar(1), foo = "foo")
     bar <- structure(foobar(2), bar = "bar")
-    expect_error(
+    (expect_error(
       with_c_foobar(vec_unchop(list(foo, bar), name_spec = "{outer}_{inner}")),
       "name specification"
-    )
+    ))
     # Used to be an error about `ptype`
-    expect_error(
+    (expect_error(
       with_c_foobar(vec_unchop(list(foobar(1)), ptype = "")),
       class = "vctrs_error_incompatible_type"
-    )
+    ))
   })
 })
 
@@ -675,15 +702,15 @@ test_that("vec_unchop() supports numeric S3 indices", {
 })
 
 test_that("vec_unchop() does not support non-numeric S3 indices", {
-  verify_errors({
-    expect_error(
+  expect_snapshot({
+    (expect_error(
       vec_unchop(list(1), list(factor("x"))),
       class = "vctrs_error_subscript_type"
-    )
-    expect_error(
+    ))
+    (expect_error(
       vec_unchop(list(1), list(foobar(1L))),
       class = "vctrs_error_subscript_type"
-    )
+    ))
   })
 })
 
@@ -702,19 +729,19 @@ test_that("can ignore names in `vec_unchop()` by providing a `zap()` name-spec (
     c(2L, 1L, 3L)
   )
 
-  verify_errors({
-    expect_error(
+  expect_snapshot({
+    (expect_error(
       vec_unchop(list(a = c(b = letters), b = 3L), name_spec = zap()),
       class = "vctrs_error_incompatible_type"
-    )
-    expect_error(
+    ))
+    (expect_error(
       vec_unchop(
         list(a = c(foo = 1:2), b = c(bar = "")),
         indices = list(2:1, 3),
         name_spec = zap()
       ),
       class = "vctrs_error_incompatible_type"
-    )
+    ))
   })
 })
 
@@ -773,44 +800,3 @@ test_that("vec_unchop() fails if foreign classes are not homogeneous and there i
     class = "vctrs_error_incompatible_type"
   )
 })
-
-test_that("vec_unchop() has informative error messages", {
-  verify_output(test_path("error", "test-unchop.txt"), {
-    "# vec_unchop() errors on unsupported location values"
-    vec_unchop(list(1, 2), list(c(1, 2), 0))
-    vec_unchop(list(1), list(-1))
-
-    "# vec_unchop() fails with complex foreign S3 classes"
-    x <- structure(foobar(1), attr_foo = "foo")
-    y <- structure(foobar(2), attr_bar = "bar")
-    vec_unchop(list(x, y))
-
-    "# vec_unchop() fails with complex foreign S4 classes"
-    joe <- .Counts(c(1L, 2L), name = "Joe")
-    jane <- .Counts(3L, name = "Jane")
-    vec_unchop(list(joe, jane))
-
-    "# vec_unchop() falls back for S4 classes with a registered c() method"
-    joe1 <- .Counts(c(1L, 2L), name = "Joe")
-    joe2 <- .Counts(3L, name = "Joe")
-    vec_unchop(list(joe1, 1, joe2), list(c(1, 2), 3, 4))
-
-    "# vec_unchop() fallback doesn't support `name_spec` or `ptype`"
-    foo <- structure(foobar(1), foo = "foo")
-    bar <- structure(foobar(2), bar = "bar")
-    with_c_foobar(vec_unchop(list(foo, bar), name_spec = "{outer}_{inner}"))
-    with_c_foobar(vec_unchop(list(foobar(1)), ptype = ""))
-
-    "# vec_unchop() does not support non-numeric S3 indices"
-    vec_unchop(list(1), list(factor("x")))
-    vec_unchop(list(1), list(foobar(1L)))
-
-    "# can ignore names in `vec_unchop()` by providing a `zap()` name-spec (#232)"
-    vec_unchop(list(a = c(b = letters), b = 3L), name_spec = zap())
-    vec_unchop(
-        list(a = c(foo = 1:2), b = c(bar = "")),
-        indices = list(2:1, 3),
-        name_spec = zap()
-      )
-  })
-})
